﻿--[[---------------------------------------------------------------------------

Copyright (c) 2008 by K. Scott Piel 
All Rights Reserved

E-mail: < kscottpiel@gmail.com >
Web:    < http://www.scottpiel.com >

This file is part of nUI.

    nUI is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    nUI is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with nUI.  If not, see <http://www.gnu.org/licenses/>.
	
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE 
USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

--]]---------------------------------------------------------------------------
	
-------------------------------------------------------------------------------
-- this function massages the user configuration to ensure all the pieces we
-- have to have are present, the string case is what we expect, numbers are
-- numbers, etc. It also allows us to easily add new options to existing
-- configurations without having to parse them, etc.

function nUI:Config_Init()

	if not nUI_Config then nUI_Config = {}; end
	if not nUI_Config.Colors then nUI_Config.Colors = {}; end
	if not nUI_Config.Colors.OOR then nUI_Config.Colors.OOR = {}; end
	if not nUI_Config.Colors.OOM then nUI_Config.Colors.OOM = {}; end
	if not nUI_Config.Colors.CDC1 then nUI_Config.Colors.CDC1 = {}; end
	if not nUI_Config.Colors.CDC2 then nUI_Config.Colors.CDC2 = {}; end

	nUI_Config.version        = nUI_Config.version or nil;
	nUI_Config.hidden         = strlower( nUI_Config.hidden or "no" );
	nUI_Config.debug          = tonumber( nUI_Config.debug or 0 );
	nUI_Config.bag_scale      = tonumber( nUI_Config.bag_scale or 0.9 );
	nUI_Config.buttonbag      = strlower( nUI_Config.buttonbag or "yes" );
	nUI_Config.map_coords     = strlower( nUI_Config.map_coords or "yes" );
	nUI_Config.chatfont_size  = tonumber( nUI_Config.chatfont_size or 12 );
	nUI_Config.unit_anim      = strlower( nUI_Config.unit_anim or "anim" );
	nUI_Config.party_pets     = strlower( nUI_Config.party_pets or "yes" );
	nUI_Config.party_targets  = strlower( nUI_Config.party_targets or "yes" );
	nUI_Config.party_ptargets = strlower( nUI_Config.party_ptargets or "yes" );
	nUI_Config.auto_ding      = strlower( nUI_Config.auto_ding or "yes" );
	nUI_Config.yOfs           = tonumber( nUI_Config.yOfs or 0 );
	nUI_Config.scale          = nUI_Config.scale or nil;
	nUI_Config.minimap_zoom   = strlower( nUI_Config.minimap_zoom or "show" );
	nUI_Config.console_fade   = strlower( nUI_Config.console_fade or "yes" );
	nUI_Config.clock          = tonumber( nUI_Config.clock or 12 );
	nUI_Config.tooltips       = strlower( nUI_Config.tooltips or "owner" );

	nUI_Config.Colors.OOR.r   = tonumber( nUI_Config.Colors.OOR.r or 1 );
	nUI_Config.Colors.OOR.g   = tonumber( nUI_Config.Colors.OOR.g or 0 );
	nUI_Config.Colors.OOR.b   = tonumber( nUI_Config.Colors.OOR.b or 0 );
	nUI_Config.Colors.OOM.r   = tonumber( nUI_Config.Colors.OOM.r or 0 );
	nUI_Config.Colors.OOM.g   = tonumber( nUI_Config.Colors.OOM.g or 0 );
	nUI_Config.Colors.OOM.b   = tonumber( nUI_Config.Colors.OOM.b or 1 );
	nUI_Config.Colors.CDC1.r  = tonumber( nUI_Config.Colors.CDC1.r or 1 );
	nUI_Config.Colors.CDC1.g  = tonumber( nUI_Config.Colors.CDC1.g or 1 );
	nUI_Config.Colors.CDC1.b  = tonumber( nUI_Config.Colors.CDC1.b or 0 );
	nUI_Config.Colors.CDC2.r  = tonumber( nUI_Config.Colors.CDC2.r or 1 );
	nUI_Config.Colors.CDC2.g  = tonumber( nUI_Config.Colors.CDC2.g or 0.6 );
	nUI_Config.Colors.CDC2.b  = tonumber( nUI_Config.Colors.CDC2.b or 0.6 );
	
	----------------------------------------------------------------------
	-- version checking... this allows us to force changes into the
	-- config based on what version of nUI the player is now using versus
	-- the one recorded in the config file
	----------------------------------------------------------------------
	
	-- if this is the first time the user has loaded nUI, then we can
	-- be certain that our default config is what we're working with
	
	local cur_version = strlower( nUI.version );
	
	if not nUI_Config.version then
	
		nUI_Config.version = cur_version;
		
	-- otherwise, if the versions are not equal...
	
	elseif cur_version ~= nUI_Config.version then
		
		-- warn the user if they go backwards in versions
		
		if cur_version < nUI_Config.version then
			
			DEFAULT_CHAT_FRAME:AddMessage( "nUI: Warning! The version of nUI you are running [|cFF00FFFF"..cur_version.."|r] is older than the version of your configuration file [|cFF00FF00"..nUI_Config.version.."|r]", 1, 0, 0 );

		-- nUI is newer than the config... so make needed changes
		
		else

			while nUI_Config.version < cur_version do
				
				local cfg_version = nUI_Config.version;
				
				if cfg_version < "0.0.9 (alpha)" then
					
					nUI:fixConfig_009a();
					
				elseif cfg_version < "0.1.0 (alpha)" then
					
					nUI:fixConfig_010a();
					
				elseif cfg_version < "0.1.2 (alpha)" then
					
					nUI:fixConfig_012a();
					
				elseif cfg_version < "0.1.5 (alpha)" then
					
					nUI:fixConfig_015a();
					
				elseif cfg_version < "0.1.6 (alpha)" then
					
					nUI:fixConfig_016a();
					
				elseif cfg_version < "0.1.8 (alpha)" then
					
					nUI:fixConfig_018a();
					
				elseif cfg_version < "0.1.9 (alpha)" then
					
					nUI:fixConfig_019a();
					
				else
				
					nUI_Config.version = cur_version;
					
				end
				
			end
			
		end
	end
	
end

-------------------------------------------------------------------------------
-- fix configuration files that pre-date 0.0.9 alpha

function nUI:fixConfig_009a()

	nUI_Config.version = "0.0.9 (alpha)";

	-- remove ButtonBag.Button configuration entries resulting from the
	-- bug that was picking up Gatherer and Cartographer minimap notes
	
	local new_buttons = {};
	
	for i in pairs( nUI_Config.ButtonBag.Buttons ) do
		
		local matched = false;
		
		for j in pairs( nUI_ButtonBag.ExcludePatterns ) do			
			if i:match( nUI_ButtonBag.ExcludePatterns[j] ) then
				matched = true;				
			end
		end
		
		if not matched then
			new_buttons[i] = nUI_Config.ButtonBag.Buttons[i];
		end				
	end
	
	nUI_Config.ButtonBag.Buttons = new_buttons;
	
end

-------------------------------------------------------------------------------
-- fix configuration files that pre-date 0.1.0 alpha

function nUI:fixConfig_010a()

	nUI_Config.version = "0.1.0 (alpha)";

	-- the button bag config has been moved into its own saved variable
	
	if nUI_Config.ButtonBag then
		nUI_ButtonBagCfg     = nUI_Config.ButtonBag;
		nUI_Config.ButtonBag = nil;
	end
	
	-- same for the movers config
	
	if nUI_Config.Movers then
		nUI_Movers = nUI_Config.Movers;
		nUI_Config.Movers = nil;
	end
	
	-- the sizing of the buffs changed, so the y offset of main unit buffs
	-- needed a tweak
	
	if nUI_UnitLayouts then
		
		if nUI_UnitLayouts["player"] then
			nUI_UnitLayouts["player"].BuffInfo.Buffs.yOfs = 160;
		end
		
		if nUI_UnitLayouts["pet"] then
			nUI_UnitLayouts["pet"].BuffInfo.Buffs.yOfs = 160;
		end
		
		if nUI_UnitLayouts["target"] then
			nUI_UnitLayouts["target"].BuffInfo.Buffs.yOfs = 160;
		end
		
		if nUI_UnitLayouts["targettarget"] then
			nUI_UnitLayouts["targettarget"].BuffInfo.Buffs.yOfs = 160;
		end		
	end	
end

-------------------------------------------------------------------------------
-- fix configuration files that pre-date 0.1.2 alpha

function nUI:fixConfig_012a()

	nUI_Config.version = "0.1.2 (alpha)";

	-- just some tweaks to unit layouts
	
	if nUI_Layouts then
			
		if nUI_UnitLayouts["pet"] 
		and nUI_UnitLayouts["pet"].Size
		and tonumber( nUI_UnitLayouts["pet"].Size.height ) == 97 then
			nUI_UnitLayouts["pet"].Size.height = 100;
		end
	
		if nUI_UnitLayouts["pettarget"]
		and nUI_UnitLayouts["pettarget"].Size
		and tonumber( nUI_UnitLayouts["pettarget"].Size.height ) == 62 then
			nUI_UnitLayouts["pettarget"].Size.height = 64;
		end
	
		if nUI_UnitLayouts["pettarget"]
		and nUI_UnitLayouts["pettarget"].Size
		and tonumber( nUI_UnitLayouts["pettarget"].Size.width ) == 212 then
			nUI_UnitLayouts["pettarget"].Size.width = 210;
		end
	
		if nUI_UnitLayouts["targettarget"] 
		and nUI_UnitLayouts["targettarget"].Size
		and tonumber( nUI_UnitLayouts["targettarget"].Size.height ) == 97 then
			nUI_UnitLayouts["targettarget"].Size.height = 100;
		end
		
		if nUI_UnitLayouts["targettargettarget"] 
		and nUI_UnitLayouts["targettargettarget"].Size
		and tonumber( nUI_UnitLayouts["targettargettarget"].Size.height ) == 62 then
			nUI_UnitLayouts["targettargettarget"].Size.height = 64;
		end

		if nUI_UnitLayouts["targettargettarget"]
		and nUI_UnitLayouts["targettargettarget"].Size
		and tonumber( nUI_UnitLayouts["targettargettarget"].Size.width ) == 212 then
			nUI_UnitLayouts["targettargettarget"].Size.width = 210;
		end
	
	end	
	
end

-------------------------------------------------------------------------------
-- fix configuration files that pre-date 0.1.5 alpha

function nUI:fixConfig_015a()

	nUI_Config.version = "0.1.5 (alpha)";

	-- just some more tweaks to unit layouts
	
	if nUI_UnitLayouts then

		-- this was an artifact from before casting bars were moved into their own config
		
		for i in pairs( nUI_UnitLayouts ) do
			
			if nUI_UnitLayouts[i].StatusBars then
				nUI_UnitLayouts[i].StatusBars.spell = nil;
			end
		end

		-- a couple of the unit buffs still needed a horizontal tweak from v0.1.0 alpha
		
		if nUI_UnitLayouts["pet"] then
			nUI_UnitLayouts["pet"].BuffInfo.Buffs.xOfs = -880;
		end
		
		if nUI_UnitLayouts["targettarget"] then
			nUI_UnitLayouts["targettarget"].BuffInfo.Buffs.xOfs = 880;
		end		
		
		-- party units have been moved into the information panel
		
		for i=1,4 do
			
			local	id   = "party"..i;
			local	tid  = id.."target";
			local	pid  = "partypet"..i;
			local	ptid = pid.."target";
			
			nUI_UnitLayouts[id]   = nil;
			nUI_UnitLayouts[tid]  = nil;
			nUI_UnitLayouts[pid]  = nil;
			nUI_UnitLayouts[ptid] = nil;
			
		end
	end	
end

-------------------------------------------------------------------------------
-- fix configuration files that pre-date 0.1.6 alpha

function nUI:fixConfig_016a()

	nUI_Config.version = "0.1.6 (alpha)";

	if nUI_BarLayouts then
		
		nUI_BarLayouts["nUI_PetActionBarFrame"] = nil;
		nUI_BarLayouts["nUI_PossessBarFrame"] = nil;
		nUI_BarLayouts["nUI_ShapeshiftBarFrame"] = nil;
		
	end
	
	-- remove ButtonBag.Button configuration entries resulting from the
	-- bug that was picking up GatherMate minimap notes
	
	local new_buttons = {};
	
	for i in pairs( nUI_ButtonBagCfg.Buttons ) do
		
		local matched = false;
		
		for j in pairs( nUI_ButtonBag.ExcludePatterns ) do			
			if i:match( nUI_ButtonBag.ExcludePatterns[j] ) then
				matched = true;				
			end
		end
		
		if not matched then
			new_buttons[i] = nUI_ButtonBagCfg.Buttons[i];
		end				
	end
	
	nUI_ButtonBagCfg.Buttons = new_buttons;
	
end

-------------------------------------------------------------------------------
-- fix configuration files that pre-date 0.1.8 alpha

function nUI:fixConfig_018a()

	nUI_Config.version = "0.1.8 (alpha)";

	if nUI_BarLayouts then
		
		nUI_BarLayouts["nUI_MainMenuBarArtFrame"] = nil;
		nUI_BarLayouts["nUI_BonusActionBarFrame"] = nil;
		
	end
	
end

-------------------------------------------------------------------------------
-- fix configuration files that pre-date 0.1.9 alpha

function nUI:fixConfig_019a()

	nUI_Config.version = "0.1.9 (alpha)";

	if nUI_BarLayouts then
		
		nUI_BarLayouts["nUI_Bar_Bags"] = nil;
		
	end
end
